# 50-cipux-task-debian-edu.perl
my $cfg = {
    'task_api' => {

        'cipux_task_change_skel_account_password' => {
            'desc'   => "Change the password of a given skel acccount.",
            'object' => [qw(cipux_account_object)],
            'action' => 'change_object_attribute_action',
            'scope'  => 'one',
            'filter' => {
                'cipux_account.user' => {
                    'cipuxRole'   => 'role',
                    'cipuxIsRole' => 'TRUE',
                    'cipuxIsSkel' => 'TRUE',
                },
            },
            'target' => {
                'cipux_account.user' => {
                    'userPassword'    => 0,
                    'sambaLMPassword' => 1,
                    'sambaNTPassword' => 1,
                },
            },
            'changes' => { 'cipux_account.user' => 'replace', },
        },
        'cipux_task_change_user_account_password' => {
            'desc'   => "Change the password of a given user account.",
            'object' => [qw(cipux_account_object)],
            'action' => 'change_object_attribute_action',
            'scope'  => 'one',
            'filter' =>
                { 'cipux_account.user' => { 'cipuxRole' => 'user', }, },
            'target' => {
                'cipux_account.user' => {
                    'userPassword'    => 0,
                    'sambaLMPassword' => 1,
                    'sambaNTPassword' => 1,
                },
            },
            'changes' => { 'cipux_account.user' => 'replace', },
        },
    },
};

# user accounts
foreach my $m (
    qw(admins students professor teachers jradmins assistant lecturer tutor pupil)
    )
{

    # +-------------------------------------------------------------+
    # | basic user account                                          |
    # +-------------------------------------------------------------+
    my $o = $m;

    $o =~ s{s$}{}smx;
    my $task = 'cipux_task_create_' . $o . '_account';
    $cfg->{task_api}->{$task} = {
        'desc'      => "Create a $o account.",
        'object'    => [qw(cipux_account_object)],
        'action'    => 'create_object_action',
        'overwrite' => {
            'cipux_account.user' => {
                'cipuxRemark' => 'CipUX task layer',
                'cipuxRole'   => "$m",
            },
            'cipux_account.group' => {
                'cipuxRemark' => 'CipUX task layer',
                'cipuxRole'   => "$m",
            },
        },
        'task' => {
            'post' => {
                'cipux_task_add_member_to_role_account' => {
                    'mode'    => 'exception',
                    'attr_hr' => {
                        object => "$m",          # role
                        value  => '@object@',    # member
                    },
                },

                # TODO OLD
                #'mode'    => 'exception',
                #'cmd_ar'  => ['cipux_task_add_member_to_role_account'],
                #'attr_ar' => [
                #    {
                #        object => "$m",          # role
                #        value  => '@object@',    # member
                #    },
                #],
            },
        },
        'dog' => {
            'post' => {
                'cipux_dog' => 'create_homedir',
                'user'      => '@object@',
                'role'      => "$m",
            },

        },
    };

    $task = 'cipux_task_destroy_' . $o . '_account';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Destroy a $o account.",
        'object' => [qw(cipux_account_object)],
        'action' => 'destroy_object_action',
        'task'   => {
            'post' => {
                'cipux_task_remove_member_from_role_account' => {
                    'mode'    => 'exception',
                    'attr_hr' => {
                        object => "$m",          # role
                        value  => '@object@',    # member
                    },
                },

                # TODO OLD
                #'mode'    => 'exception',
                #'cmd_ar'  => ['cipux_task_remove_member_from_role_account'],
                #'attr_ar' => [
                #    {
                #        object => "$m",          # role
                #        value  => '@object@',    # member
                #    },
                #],
            },
        },
    };

    $task = 'cipux_task_list_' . $o . '_accounts';
    $cfg->{task_api}->{$task} = {
        'desc'       => "List all $o accounts.",
        'object'     => [qw(cipux_role_object)],
        'action'     => 'list_object_attribute_action',
        'scope'      => 'one',
        'filter'     => { 'cipux_role.group' => { 'cn' => "$m", }, },
        'taskobject' => "$m",
        'target'     => { 'cipux_role.group' => { 'memberUid' => 0, }, },
        'ltarget'    => 'memberUid',
    };

    $task = 'cipux_task_list_members_of_' . $o . '_account';
    $cfg->{task_api}->{$task} = {
        'desc'    => "List all members of a given $o account.",
        'object'  => [qw(cipux_account_object)],
        'action'  => 'list_object_attribute_action',
        'scope'   => 'one',
        'filter'  => { 'cipux_account.group' => { 'cipuxRole' => "$m", }, },
        'target'  => { 'cipux_account.group' => { 'memberUid' => 0, }, },
        'ltarget' => 'memberUid',
    };

# cipux_ldap_set_value -a -t cipux_account.group -y memberUid=xxx -o bibeutlin
#mod filter [&(cn=bibeutlin)(objectClass=cipuxGroup)(cipuxIsAccount=TRUE)]
    $task = 'cipux_task_add_member_to_' . $o . '_account';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Add a member to a given $o account.",
        'object' => [qw(cipux_account_object)],
        'action' => 'change_object_attribute_action',
        'scope'  => 'one',
        'filter' => { 'cipux_account.group' => { 'cipuxRole' => "$m", }, },
        'target'  => { 'cipux_account.group' => { 'memberUid' => 0, }, },
        'changes' => { 'cipux_account.group' => 'add', },
    };

    $task = 'cipux_task_remove_member_from_' . $o . '_account';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Remove a member from a given $o account.",
        'object' => [qw(cipux_account_object)],
        'action' => 'change_object_attribute_action',
        'scope'  => 'one',
        'filter' => { 'cipux_account.group' => { 'cipuxRole' => "$m", }, },
        'target'  => { 'cipux_account.group' => { 'memberUid' => 0, }, },
        'changes' => { 'cipux_account.group' => 'delete', },
    };

    $task = 'cipux_task_change_' . $o . '_account_password';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Change the password of a given $o account.",
        'object' => [qw(cipux_account_object)],
        'action' => 'change_object_attribute_action',
        'scope'  => 'one',
        'filter' => { 'cipux_account.user' => { 'cipuxRole' => "$m", }, },
        'target' => {
            'cipux_account.user' => {
                'userPassword'    => 0,
                'sambaLMPassword' => 1,
                'sambaNTPassword' => 1,
            },
        },
        'changes' => { 'cipux_account.user' => 'replace', },
    };

    # Concept of mainrole dropt in Dec 2008 in favor of multi role schema
    # However a non exclusive main role might reintroduced later
    #$task = 'cipux_task_retrieve_all_' . $o . '_account_mainrole';
    #$cfg->{task_api}->{$task} = {
    #    'desc'   => "Get lastname info from all $o accounts.",
    #    'object' => [qw(cipux_role_object)],
    #    'action' => 'list_object_attribute_action',
    #    #    #    #    #    'scope'  => 'one',
    #    'filter' => {
    #        'cipux_role.group' => {
    #            'groupType' => 'authority_group',
    #            'cn'        => "$m"
    #        },
    #    },
    #    'taskobject' => "$m",
    #    'target'     => { 'cipux_role.group' => {'memberUid'=>0,}, },
    #    'subtarget' =>
    #        { 'cipux_account.user' => [qw(cipuxRole)], },
    #    'subfilter' =>
    #        { 'cipux_account.user' => { objectClass => 'cipuxAccount' } },
    #};

    $task = 'cipux_task_change_' . $o . '_account_role';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Change the $o account role.",
        'object' => [qw(cipux_account_object)],
        'action' => 'change_object_attribute_action',
        'scope'  => 'one',
        'filter' =>
            { 'cipux_account.user' => { 'cipuxIsAccount' => 'TRUE', }, },
        'target'  => { 'cipux_account.user' => { 'cipuxRole' => 0, }, },
        'changes' => { 'cipux_account.user' => 'replace', },
    };

    $task = 'cipux_task_retrieve_all_' . $o . '_account_homedir';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Get lastname info from all $o accounts.",
        'object' => [qw(cipux_role_object)],
        'action' => 'list_object_attribute_action',
        'scope'  => 'one',
        'filter' => {
            'cipux_role.group' => {
                'groupType' => 'authority_group',
                'cn'        => "$m"
            },
        },
        'taskobject' => "$m",
        'target'     => { 'cipux_role.group' => { 'memberUid' => 0, }, },
        'ltarget'    => 'uid',
        'subtarget' => { 'cipux_account.user' => { 'homeDirectory' => 0, }, },
        'subfilter' =>
            { 'cipux_account.user' => { objectClass => 'cipuxAccount' } },
    };

    $task = 'cipux_task_retrieve_all_' . $o . '_account_firstname';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Get firstname info from all $o accounts.",
        'object' => [qw(cipux_role_object)],
        'action' => 'list_object_attribute_action',
        'scope'  => 'one',
        'filter' => {
            'cipux_role.group' => {
                'groupType' => 'authority_group',
                'cn'        => "$m"
            },
        },
        'taskobject' => "$m",
        'target'     => { 'cipux_role.group' => { 'memberUid' => 0, }, },
        'ltarget'    => 'uid',
        'subtarget' =>
            { 'cipux_account.user' => { 'cipuxFirstname' => 0, }, },
        'subfilter' =>
            { 'cipux_account.user' => { objectClass => 'cipuxAccount' } },
    };

    $task = 'cipux_task_retrieve_all_' . $o . '_account_lastname';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Get lastname info from all $o accounts.",
        'object' => [qw(cipux_role_object)],
        'action' => 'list_object_attribute_action',
        'scope'  => 'one',
        'filter' => {
            'cipux_role.group' => {
                'groupType' => 'authority_group',
                'cn'        => "$m"
            },
        },
        'taskobject' => "$m",
        'target'     => { 'cipux_role.group' => { 'memberUid' => 0, }, },
        'ltarget'    => 'uid',
        'subtarget' => { 'cipux_account.user' => { 'cipuxLastname' => 0, }, },
        'subfilter' =>
            { 'cipux_account.user' => { objectClass => 'cipuxAccount' } },
    };

    # Most used task with sub task section
    $task = 'cipux_task_retrieve_all_' . $o . '_account_lastname_firstname';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Get firstname and lastname info from all $o accounts.",
        'object' => [qw(cipux_role_object)],
        'action' => 'list_object_attribute_action',
        'scope'  => 'one',
        'filter' => {
            'cipux_role.group' => {
                'groupType' => 'authority_group',
                'cn'        => "$m"
            },
        },
        'taskobject' => "$m",
        'target'     => { 'cipux_role.group' => { 'memberUid' => 0, }, },
        'ltarget'    => 'uid',

        # This task has a sub task section 'subtarget' and 'subfilter' with
        # defines the sub task. In general it means that the above target
        # 'memberUid' will be used as a positive list against all accounts
        # queried by the subfilter. The result is in alphabetical order.
        #
        # Example:
        # 1 task gives:
        #     memberUid: user1
        #     memberUid: user2
        # 2 task gives:
        #     uid user1, cipuxFirstname: fn1, cipuxLastname: ln1
        #     uid user2, cipuxFirstname: fn2, cipuxLastname: ln2
        #     uid user3, cipuxFirstname: fn3, cipuxLastname: ln3
        # Then the line 3 will be filtered because 'uid' is not in list
        # 'memberUid' The output will be lines 1-2
        'subtarget' => {
            'cipux_account.user' => {
                'cipuxLastname'  => 0,
                'cipuxFirstname' => 0,
            },
        },
        'subfilter' =>
            { 'cipux_account.user' => { objectClass => 'cipuxAccount' } },
    };
    $task
        = 'cipux_task_retrieve_all_' 
        . $o
        . '_account_lastname_firstname_role';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Get firstname and lastname info from all $o accounts.",
        'object' => [qw(cipux_role_object)],
        'action' => 'list_object_attribute_action',
        'scope'  => 'one',
        'filter' => {
            'cipux_role.group' => {
                'groupType' => 'authority_group',
                'cn'        => "$m"
            },
        },
        'taskobject' => "$m",
        'target'     => { 'cipux_role.group' => ['memberUid'], },
        'ltarget'    => 'uid',
        'subtarget'  => {
            'cipux_account.user' => {
                cipuxLastname  => 0,
                cipuxFirstname => 0,
                cipuxRole      => 0,
            },
        },
        'subfilter' =>
            { 'cipux_account.user' => { objectClass => 'cipuxAccount' } },
    };
}

# +-------------------------------------------------------------------------+
# | basic group                                                             |
# +-------------------------------------------------------------------------+

# CipUX CAT module modality:

for my $g (
    qw(course class studygroup seminar lecture reading workshop tutorial team)
    )
{

    # +-----------------------------------------------------------------+
    # | basic group share                                               |
    # +-----------------------------------------------------------------+

    my $task = 'cipux_task_create_' . $g . '_share';
    $cfg->{task_api}->{$task} = {
        'desc'      => "Create $g share account.",
        'object'    => [qw(cipux_share_object)],
        'action'    => 'create_object_action',
        'overwrite' => {
            'cipux_share.user' => {
                'cipuxRemark' => 'CipUX task layer',
                'cipuxRole'   => $g,
            },
            'cipux_share.group' => {
                'cipuxRemark' => 'CipUX task layer',
                'cipuxRole'   => $g,
            },
        },
        'dog' => {
            'post' => {
                'cipux_dog' => 'create_homedir',
                'user'      => '@object@',
                'role'      => "$g",
            },
        },
    };
    $task = 'cipux_task_destroy_' . $g . '_share';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Destroy $g share account.",
        'object' => [qw(cipux_share_object)],
        'action' => 'destroy_object_action',
    };

    # mod filter
    # [&(uid=*)
    #   (objectClass=cipuxAccount)
    #   (cipuxIsShare=TRUE)
    #   (cipuxRole=course)]
    $task = 'cipux_task_list_' . $g . '_shares';
    $cfg->{task_api}->{$task} = {
        'desc'   => "List all $g share accounts.",
        'object' => [qw(cipux_share_object)],
        'action' => 'list_object_action',
        'filter' => {
            'cipux_share.user'  => { 'cipuxRole' => $g, },
            'cipux_share.group' => { 'cipuxRole' => $g, },
        },
        'ltarget' => 'uid',
    };

    # cipux_ldap_get_value  -t cipux_share.group -o class84 -y memberUid
    # mod filter [&(cn=class84)(objectClass=cipuxGroup)(cipuxIsShare=TRUE)
    $task = 'cipux_task_list_members_of_' . $g . '_share';
    $cfg->{task_api}->{$task} = {
        'desc'    => "List all members of a given $g share account.",
        'object'  => [qw(cipux_share_object)],
        'action'  => 'list_object_attribute_action',
        'scope'   => 'one',
        'filter'  => { 'cipux_share.group' => { 'cipuxRole' => $g, }, },
        'target'  => { 'cipux_share.group' => { 'memberUid' => 0, }, },
        'ltarget' => 'memberUid',
    };

    # cipux_ldap_set_value -a -t cipux_share.group -y memberUid=xxx -o class85
    #mod filter [&(cn=class85)(objectClass=cipuxGroup)(cipuxIsShare=TRUE)]
    $task = 'cipux_task_add_member_to_' . $g . '_share';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Add a  member to a given $g share account.",
        'object' => [qw(cipux_share_object)],
        'action' => 'change_object_attribute_action',
        'scope'  => 'one',
        'filter' => { 'cipux_share.group' => { 'cipuxRole' => $g, }, },
        'target'  => { 'cipux_share.group' => { 'memberUid' => 0, }, },
        'changes' => { 'cipux_share.group' => 'add', },
    };

    $task = 'cipux_task_remove_member_from_' . $g . '_share';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Remove a member from a given $g share account.",
        'object' => [qw(cipux_share_object)],
        'action' => 'change_object_attribute_action',
        'scope'  => 'one',
        'filter' => { 'cipux_share.group' => { 'cipuxRole' => $g, }, },
        'target'  => { 'cipux_share.group' => { 'memberUid' => 0, }, },
        'changes' => { 'cipux_share.group' => 'delete', },
    };

    # 'cipux_task_remove_user_from_course_share',
    # 'cipux_task_list_user_nonmembers_of_course_share',

    $task = 'cipux_task_change_' . $g . '_share_password';
    $cfg->{task_api}->{$task} = {
        'desc'   => "Change the password of a given $g account.",
        'object' => [qw(cipux_share_object)],
        'action' => 'change_object_attribute_action',
        'scope'  => 'one',
        'filter' => { 'cipux_share.user' => { 'cipuxRole' => "$g", }, },
        'target' => {
            'cipux_share.user' => {
                'userPassword'    => 0,
                'sambaLMPassword' => 1,
                'sambaNTPassword' => 1,
            },
        },
        'changes' => { 'cipux_share.user' => 'replace', },
    };
}

# ALIAS task
my $alias = 'cipux_task_change_user_account_password_interactive';
my $task  = 'cipux_task_change_user_account_password';
$cfg->{task_api}->{$alias} = $cfg->{task_api}->{$task};
$alias                     = 'cipux_task_change_own_password_interactive';
$cfg->{task_api}->{$alias} = $cfg->{task_api}->{$task};

undef $task;
undef $alias;

return $cfg;
